package ru.waptaxi.wtwebclient.jettyserv.vaadin.testfocus;

import com.vaadin.data.provider.ListDataProvider;
import com.vaadin.shared.ui.grid.HeightMode;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Grid;
import com.vaadin.ui.themes.ValoTheme;
import ru.waptaxi.wtwebclient.jettyserv.vaadin.testfocus.gridline.GridLine;

import java.util.LinkedList;
import java.util.concurrent.ConcurrentHashMap;

public class TestFocusGrid {

    private Grid<GridLine> grid;

    ConcurrentHashMap<Integer, GridLine> mapLineNumberToGridLine = new ConcurrentHashMap<>();
    {
        mapLineNumberToGridLine.put(1, new GridLine(this));
        mapLineNumberToGridLine.put(2, new GridLine(this));
        mapLineNumberToGridLine.put(3, new GridLine(this));
        mapLineNumberToGridLine.put(4, new GridLine(this));
        mapLineNumberToGridLine.put(5, new GridLine(this));
        mapLineNumberToGridLine.put(6, new GridLine(this));
        mapLineNumberToGridLine.put(7, new GridLine(this));
        mapLineNumberToGridLine.put(8, new GridLine(this));
        mapLineNumberToGridLine.put(9, new GridLine(this));
        mapLineNumberToGridLine.put(10, new GridLine(this));

    }

    private TestFocusVaadinUI currentUI;

    // Entities list that we connect to DataProvider
    private LinkedList<GridLine> dataList = new LinkedList<>();


    public TestFocusGrid(TestFocusVaadinUI currentUI) {
        this.currentUI = currentUI;

        grid = new Grid<>();
        grid.setSizeFull();
        grid.setHeightMode(HeightMode.ROW);
        grid.setHeightByRows(10);
        grid.setStyleName(ValoTheme.TABLE_BORDERLESS);

        // Columns
        grid.addComponentColumn(GridLine::getCityComboBoxComponent).setCaption("City").setSortable(false).setWidth(150);
        grid.addComponentColumn(GridLine::getStreetComboBoxComponent).setCaption("Street").setSortable(false).setWidth(150);

        // DataProvider
        ListDataProvider<GridLine> dataProvider = new ListDataProvider<>(dataList);
        grid.setDataProvider(dataProvider);

        reloadDataList();

        dataList.get(0).getCityComboBoxT().reloadDataList();
        dataList.get(0).getCityComboBoxComponent().focus();



    }


    /**
     * Reload Grid's dataList
     */
    public void reloadDataList() {

        //Mixing two lines for visual
        Double random1 = Math.random() * 10; // 0..9
        int index1 = random1.intValue() + 1; // 1..10
        Double random2 = Math.random() * 10; // 0..9
        int index2 = random2.intValue() + 1; // 1..10
        GridLine gridLine1 = mapLineNumberToGridLine.get(index1);
        GridLine gridLine2 = mapLineNumberToGridLine.get(index2);
        mapLineNumberToGridLine.put(index1, gridLine2);
        mapLineNumberToGridLine.put(index2, gridLine1);

        dataList.clear();
        for (int i = 1; i < 11; i++) {
            dataList.add(mapLineNumberToGridLine.get(i));
        }
        grid.getDataProvider().refreshAll();
    }


    public Grid<GridLine> getGrid() {
        return grid;
    }


    public void moveFocusFromStreetToNextLineOnCity(ComboBox<String> streetComboBox) {
        for (int i = 0; i < dataList.size(); i++) {
            GridLine gridLine = dataList.get(i);
            if (gridLine.getStreetComboBoxComponent().equals(streetComboBox)) {
                // If not last line then move focus on next line on City
                // else move focus on first line on City
                if (i < dataList.size() - 1){
                    dataList.get(i + 1).getCityComboBoxT().reloadDataList();
                    dataList.get(i + 1).getCityComboBoxComponent().focus();
                } else {
                    dataList.get(0).getCityComboBoxT().reloadDataList();
                    dataList.get(0).getCityComboBoxComponent().focus();
                }
            }
        }
    }

    public void moveFocusFromCityToStreet(ComboBox<String> cityComboBox) {
        for (int i = 0; i < dataList.size(); i++) {
            GridLine gridLine = dataList.get(i);
            if (gridLine.getCityComboBoxComponent().equals(cityComboBox)) {
                gridLine.getStreetComboBoxT().reloadDataList();
                gridLine.getStreetComboBoxComponent().focus();
            }
        }

    }

    public TestFocusVaadinUI getCurrentUI() {
        return currentUI;
    }
}
