package ru.waptaxi.wtwebclient.jettyserv.vaadin.testfocus;

import com.vaadin.annotations.Push;
import com.vaadin.annotations.Theme;
import com.vaadin.annotations.Title;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.server.*;
import com.vaadin.shared.communication.PushMode;
import com.vaadin.shared.ui.ui.Transport;
import com.vaadin.ui.*;

import javax.servlet.annotation.WebServlet;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

@Title("WTWebClient")
@Theme("wtvalo")
@Push(value = PushMode.AUTOMATIC, transport = Transport.WEBSOCKET)
public class TestFocusVaadinUI extends UI {

    private static final long serialVersionUID = -3823711621080689306L;


    // Trigger for another session to be able to catch click event
    public static volatile Object buttonClickedUIInstance = null;

    private TestFocusGrid testFocusGrid;
    private Label label;
    public Label labelFocusInfo;



    @WebServlet(value = "/*", asyncSupported = true)
    @VaadinServletConfiguration(productionMode = false, ui = TestFocusVaadinUI.class)
    public static class Servlet extends VaadinServlet {

    }


    @Override
    protected void init(VaadinRequest request) {

        testFocusGrid = new TestFocusGrid(this);
        label = new Label();
        labelFocusInfo = new Label();

        Button button001 = new Button("Set trigger to fire in another UI");
        button001.addClickListener(event -> buttonClickedUIInstance = getUI());

        // Main vertical layout
        VerticalLayout vl = new VerticalLayout();
        vl.addComponent(new Label("UI instance: " + getUI()));
        vl.addComponent(testFocusGrid.getGrid());
        vl.addComponent(button001);
        vl.addComponent(label);
        vl.addComponent(labelFocusInfo);


        setContent(vl);

        // Don't do that in real projects - it may produce threads leak
        generateSimpleScheduler();

    }


    private void generateSimpleScheduler() {
        UI currentUI = getUI();
        // Scheduler to check buttonClickedUIInstance is not null
        ScheduledExecutorService service = Executors.newSingleThreadScheduledExecutor();
        service.scheduleWithFixedDelay(new Runnable() {
            @Override
            public void run() {
                // Stopping scheduler UI closed. May be wrong, not tested
                if (currentUI == null || currentUI.isClosing()){
                    service.shutdown();
                } else {
                    if (buttonClickedUIInstance != null) {
                        if (buttonClickedUIInstance == currentUI) {
                            label.setValue("Catched 'buttonClickedUIInstance' from same thread");
                        } else {
                            label.setValue("Catched 'buttonClickedUIInstance' from other thread and cleaned\n" +
                                    "Reload Grid datalist with mixing lines");
                            buttonClickedUIInstance = null;
                            testFocusGrid.reloadDataList();
                        }
                    } else {
                        label.setValue("'buttonClickedUIInstance' is null");
                    }
                }
            }
        }, 0, 1, TimeUnit.SECONDS);
    }


}
