package ru.waptaxi.wtwebclient.jettyserv.vaadin.testfocus.gridline;


import com.vaadin.data.HasValue;
import com.vaadin.data.provider.ListDataProvider;
import com.vaadin.event.FieldEvents;
import com.vaadin.shared.Registration;
import com.vaadin.ui.ComboBox;

import com.vaadin.ui.ItemCaptionGenerator;
import com.vaadin.ui.themes.ValoTheme;
import org.apache.commons.lang3.RandomStringUtils;
import java.util.LinkedList;


public class CityComboBoxT {

    private GridLine parentGridLine;

    private ComboBox<String> comboBox;

    private LinkedList<String> dataList = new LinkedList<>();

    private HasValue.ValueChangeListener<String> valueChangeListener = createValueChangeListener();
    private FieldEvents.FocusListener focusListener = getFocusListener();
    private FieldEvents.BlurListener blurListener = getBlurListener();

    // Registration for future unregister if needed
    private Registration registrationValueChangeListener = null;




    public CityComboBoxT(GridLine parentGridLine) {
        this.parentGridLine = parentGridLine;

        comboBox = new ComboBox<>();
        comboBox.setSizeFull();
        comboBox.setStyleName(ValoTheme.COMBOBOX_BORDERLESS);
        comboBox.setEmptySelectionAllowed(true);

        // DataProvider
        comboBox.setItemCaptionGenerator((ItemCaptionGenerator<String>) String::toString);
        ListDataProvider<String> dataProvider = new ListDataProvider<>(dataList);
        comboBox.setDataProvider(dataProvider);

        registrationValueChangeListener = comboBox.addValueChangeListener(valueChangeListener);
        comboBox.addFocusListener(focusListener);
        comboBox.addBlurListener(blurListener);

        reloadDataList();

    }


    /**
     * ValueChangeListener
     */
    private HasValue.ValueChangeListener<String> createValueChangeListener() {
        return (HasValue.ValueChangeListener<String>) event -> {
            parentGridLine.getParentFocusGrid().moveFocusFromCityToStreet(comboBox);
        };
    }


    /**
     * FocusListener
     */
    private FieldEvents.FocusListener getFocusListener() {
        return (FieldEvents.FocusListener) event -> {
            parentGridLine.getParentFocusGrid().getCurrentUI().labelFocusInfo.setValue("Focus on CityComboBox: " + comboBox);
        };
    }


    /**
     * BlurListener
     */
    private FieldEvents.BlurListener getBlurListener() {
        return (FieldEvents.BlurListener) event -> {
            parentGridLine.getParentFocusGrid().getCurrentUI().labelFocusInfo.setValue("Focus moved from CityComboBox: " + comboBox);
        };
    }


    public ComboBox<String> getComboBox() {
        return comboBox;
    }


    /**
     * Updating dataList
     */
    public void reloadDataList() {
        registrationValueChangeListener.remove();

        comboBox.setValue(null); // To fire event when select item
        dataList.clear();


        for (int i = 0; i < 100; i++) {
            String cityName = "city_" + RandomStringUtils.random(5, true, true);
            dataList.add(cityName);
        }

        comboBox.getDataProvider().refreshAll();

        registrationValueChangeListener = comboBox.addValueChangeListener(valueChangeListener);

    }


}
