package ru.waptaxi.wtwebclient.jettyserv.vaadin.testfocus.gridline;


import com.vaadin.data.HasValue;
import com.vaadin.data.provider.ListDataProvider;
import com.vaadin.event.FieldEvents;
import com.vaadin.shared.Registration;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.ItemCaptionGenerator;
import com.vaadin.ui.themes.ValoTheme;
import org.apache.commons.lang3.RandomStringUtils;

import java.util.ArrayList;
import java.util.LinkedList;

public class StreetComboBoxT {

    private GridLine parentGridLine;

    private ComboBox<String> comboBox;
    private LinkedList<String> dataList = new LinkedList<>();


    private HasValue.ValueChangeListener<String> valueChangeListener = createValueChangeListener();
    private FieldEvents.FocusListener focusListener = createFocusListener();
    private FieldEvents.BlurListener blurListener = createBlurListener();

    private Registration registrationValueChangeListener = null;



    public StreetComboBoxT(GridLine parentGridLine){
        this.parentGridLine = parentGridLine;

        comboBox = new ComboBox<>();

        comboBox.setSizeFull();
        comboBox.setStyleName(ValoTheme.COMBOBOX_BORDERLESS);
        comboBox.setEmptySelectionAllowed(true);

        comboBox.setItemCaptionGenerator((ItemCaptionGenerator<String>) String::toString);
        ListDataProvider<String> dataProvider = new ListDataProvider<>(dataList);
        comboBox.setDataProvider(dataProvider);

        registrationValueChangeListener = comboBox.addValueChangeListener(valueChangeListener);
        comboBox.addFocusListener(focusListener);
        comboBox.addBlurListener(blurListener);

        reloadDataList();
    }


    private FieldEvents.FocusListener createFocusListener() {
        return (FieldEvents.FocusListener) event -> {
            parentGridLine.getParentFocusGrid().getCurrentUI().labelFocusInfo.setValue("Focus on StreetComboBox:" + comboBox);
        };
    }


    private FieldEvents.BlurListener createBlurListener() {
        return (FieldEvents.BlurListener) event -> {
            parentGridLine.getParentFocusGrid().getCurrentUI().labelFocusInfo.setValue("Focus moved from StreetComboBox:" + comboBox);
        };
    }


    private HasValue.ValueChangeListener<String> createValueChangeListener() {
        return (HasValue.ValueChangeListener<String>) event -> {
            parentGridLine.getParentFocusGrid().moveFocusFromStreetToNextLineOnCity(comboBox);
        };
    }

    public ComboBox<String> getComboBox() {
        return comboBox;
    }

    public void reloadDataList() {
        registrationValueChangeListener.remove();
        comboBox.clear();
        dataList.clear();
        for (int i = 0; i < 100; i++) {
            String streetName = "street_" + RandomStringUtils.random(5, true, true);
            dataList.add(streetName);
        }
        comboBox.getDataProvider().refreshAll();
        registrationValueChangeListener = comboBox.addValueChangeListener(valueChangeListener);
    }
}
